@@ -904,14 +904,30 @@ gkm_wrap_prompt_init (GkmWrapPrompt *self)
 }
 
 static void
-gkm_wrap_prompt_finalize (GObject *obj)
+gkm_wrap_prompt_clear_prompt_data (GkmWrapPrompt *self)
 {
-	GkmWrapPrompt *self = GKM_WRAP_PROMPT (obj);
-
 	if (self->destroy_data && self->prompt_data)
 		(self->destroy_data) (self->prompt_data);
 	self->destroy_data = NULL;
 	self->prompt_data = NULL;
+}
+
+static void
+gkm_wrap_prompt_set_prompt_data (GkmWrapPrompt *self,
+				 gpointer prompt_data,
+				 GDestroyNotify destroy_data)
+{
+	gkm_wrap_prompt_clear_prompt_data (self);
+	self->destroy_data = destroy_data;
+	self->prompt_data = prompt_data;
+}
+
+static void
+gkm_wrap_prompt_finalize (GObject *obj)
+{
+	GkmWrapPrompt *self = GKM_WRAP_PROMPT (obj);
+
+	gkm_wrap_prompt_clear_prompt_data (self);
 
 	while (!g_queue_is_empty(&self->pool))
 		g_free (g_queue_pop_head (&self->pool));
@@ -934,7 +950,7 @@ gkm_wrap_prompt_class_init (GkmWrapPromptClass *klass)
 typedef struct _CredentialPrompt {
 	GArray *template;
 	CK_ULONG n_template;
-	const gchar *password;
+	gchar *password;
 } CredentialPrompt;
 
 static void
@@ -942,6 +958,7 @@ credential_prompt_free (gpointer user_data)
 {
 	CredentialPrompt *data = user_data;
 	g_array_free (data->template, TRUE);
+	egg_secure_strfree (data->password);
 	g_slice_free (CredentialPrompt, data);
 }
 
@@ -977,8 +994,8 @@ gkm_wrap_prompt_for_credential (CK_FUNCTION_LIST_PTR module, CK_SESSION_HANDLE s
 	                     NULL);
 
 	/* Build up the prompt */
-	self->prompt_data = data = g_slice_new0 (CredentialPrompt);
-	self->destroy_data = credential_prompt_free;
+	data = g_slice_new0 (CredentialPrompt);
+	gkm_wrap_prompt_set_prompt_data (self, data, credential_prompt_free);
 	self->module = module;
 	self->session = session;
 	self->object = object;
@@ -1017,6 +1034,7 @@ gkm_wrap_prompt_do_credential (GkmWrapPrompt *self, CK_ATTRIBUTE_PTR *template,
 	attrs = get_attributes_from_object (self, &n_attrs);
 	g_return_val_if_fail (attrs, FALSE);
 
+	egg_secure_strfree (data->password);
 	data->password = NULL;
 
 	if (self->iteration == 0) {
@@ -1028,6 +1046,7 @@ gkm_wrap_prompt_do_credential (GkmWrapPrompt *self, CK_ATTRIBUTE_PTR *template,
 	}
 
 	if (!data->password) {
+		const char *password;
 		setup_unlock_prompt (self, attrs, n_attrs, self->iteration == 1);
 
 		/* Now load up the unlock options into the prompt*/
@@ -1039,14 +1058,15 @@ gkm_wrap_prompt_do_credential (GkmWrapPrompt *self, CK_ATTRIBUTE_PTR *template,
 
 		++(self->iteration);
 
-		data->password = gkm_wrap_prompt_request_password (self);
-		if (data->password == NULL) {
+		password = gkm_wrap_prompt_request_password (self);
+		if (password == NULL) {
 			if (error != NULL) {
 				g_warning ("couldn't prompt for password: %s", egg_error_message (error));
 				g_error_free (error);
 			}
 			return FALSE;
 		}
+		data->password = egg_secure_strdup (password);
 	}
 
 	/* Truncate any extra options off the end of template */
@@ -1173,7 +1193,7 @@ gkm_wrap_prompt_do_init_pin (GkmWrapPrompt *self, CK_RV last_result,
                              CK_UTF8CHAR_PTR *pin, CK_ULONG *n_pin)
 {
 	CK_TOKEN_INFO tinfo;
-	const gchar *password;
+	gchar *password;
 
 	g_assert (GKM_IS_WRAP_PROMPT (self));
 	g_assert (self->module);
@@ -1185,11 +1205,12 @@ gkm_wrap_prompt_do_init_pin (GkmWrapPrompt *self, CK_RV last_result,
 
 	setup_init_token (self, &tinfo);
 
-	password = gkm_wrap_prompt_request_password (self);
+	password = (char *)gkm_wrap_prompt_request_password (self);
 	if (password == NULL)
 		return FALSE;
 
-	self->prompt_data = (gpointer)password;
+	g_assert (self->destroy_data == NULL);
+	gkm_wrap_prompt_set_prompt_data (self, password, NULL);
 	*pin = (gpointer)password;
 	*n_pin = strlen (password);
 	return TRUE;
@@ -1314,8 +1335,9 @@ gkm_wrap_prompt_for_set_pin (CK_FUNCTION_LIST_PTR module, CK_SESSION_HANDLE sess
 	/* Build up the prompt */
 	self->module = module;
 	self->session = session;
-	self->destroy_data = set_pin_prompt_free;
-	self->prompt_data = g_slice_new0 (SetPinPrompt);
+	gkm_wrap_prompt_set_prompt_data (self,
+					 g_slice_new0 (SetPinPrompt),
+					 set_pin_prompt_free);
 
 	return self;
 }
@@ -1438,7 +1460,7 @@ static gboolean
 login_prompt_do_specific (GkmWrapPrompt *self, CK_RV last_result,
                           CK_UTF8CHAR_PTR *pin, CK_ULONG *n_pin)
 {
-	const gchar *password = NULL;
+	gchar *password = NULL;
 	CK_ATTRIBUTE_PTR attrs;
 	CK_ULONG n_attrs;
 
@@ -1451,6 +1473,9 @@ login_prompt_do_specific (GkmWrapPrompt *self, CK_RV last_result,
 	if (self->iteration == 0) {
 		++(self->iteration);
 		password = auto_unlock_lookup_object (attrs, n_attrs);
+		if (password)
+			gkm_wrap_prompt_set_prompt_data (self, password,
+							 (GDestroyNotify)egg_secure_strfree);
 
 	} else if (self->iteration == 1 && last_result == CKR_PIN_INCORRECT) {
 		auto_unlock_remove_object (attrs, n_attrs);
@@ -1459,12 +1484,12 @@ login_prompt_do_specific (GkmWrapPrompt *self, CK_RV last_result,
 	if (!password) {
 		setup_unlock_prompt (self, attrs, n_attrs, self->iteration == 1);
 
-		password = gkm_wrap_prompt_request_password (self);
+		password = (char *)gkm_wrap_prompt_request_password (self);
 		if (password == NULL)
 			return FALSE;
+		gkm_wrap_prompt_set_prompt_data (self, password, NULL);
 	}
 
-	self->prompt_data = (gpointer)password;
 	*pin = (guchar *)password;
 	*n_pin = strlen (password);
 	return TRUE;
@@ -1477,7 +1502,8 @@ login_prompt_done_specific (GkmWrapPrompt *self, CK_RV call_result)
 	CK_ULONG n_attrs;
 
 	g_assert (GKM_IS_WRAP_PROMPT (self));
-	g_assert (self->destroy_data == NULL);
+	g_assert (self->destroy_data == NULL ||
+		  self->destroy_data == (GDestroyNotify)egg_secure_strfree);
 
 	/* Possibly save away auto unlock */
 	if (call_result == CKR_OK && auto_unlock_should_attach (self)) {
@@ -1510,7 +1536,7 @@ login_prompt_do_user (GkmWrapPrompt *self, CK_RV last_result,
                        CK_UTF8CHAR_PTR *pin, CK_ULONG *n_pin)
 {
 	CK_TOKEN_INFO tinfo;
-	const gchar *password = NULL;
+	gchar *password = NULL;
 
 	g_assert (GKM_IS_WRAP_PROMPT (self));
 	g_assert (self->module);
@@ -1523,6 +1549,9 @@ login_prompt_do_user (GkmWrapPrompt *self, CK_RV last_result,
 	if (self->iteration == 0) {
 		++(self->iteration);
 		password = auto_unlock_lookup_token (&tinfo);
+		if (password)
+			gkm_wrap_prompt_set_prompt_data (self, password,
+							 (GDestroyNotify)egg_secure_strfree);
 
 	} else if (self->iteration == 1 && last_result == CKR_PIN_INCORRECT) {
 		auto_unlock_remove_token (&tinfo);
@@ -1531,12 +1560,12 @@ login_prompt_do_user (GkmWrapPrompt *self, CK_RV last_result,
 	if (!password) {
 		setup_unlock_token (self, &tinfo);
 
-		password = gkm_wrap_prompt_request_password (self);
+		password = (char *)gkm_wrap_prompt_request_password (self);
 		if (password == NULL)
 			return FALSE;
+		gkm_wrap_prompt_set_prompt_data (self, password, NULL);
 	}
 
-	self->prompt_data = (gpointer)password;
 	*pin = (guchar *)password;
 	*n_pin = strlen (password);
 	return TRUE;
@@ -1548,7 +1577,8 @@ login_prompt_done_user (GkmWrapPrompt *self, CK_RV call_result)
 	CK_TOKEN_INFO tinfo;
 
 	g_assert (GKM_IS_WRAP_PROMPT (self));
-	g_assert (self->destroy_data == NULL);
+	g_assert (self->destroy_data == NULL ||
+		  self->destroy_data == (GDestroyNotify)egg_secure_strfree);
 
 	/* Save the options, and possibly auto unlock */
 	if (call_result == CKR_OK && auto_unlock_should_attach (self)) {
